#include <stdio.h>

#include "raylib.h"

typedef struct Player {
    Vector2 screen;
    float radius;
    Vector2 coord;
} Player;

typedef struct Map {
    const Texture texture;
    const Vector2 screen;
    const Rectangle box;
} Map;

int main(void)
{
    Vector2 window_size = { 800.0f, 600.0f };

    Camera2D camera = {
        .target = {0},
        .offset = (Vector2) {window_size.x/2, window_size.y/2},
        .rotation = 0.0f,
        .zoom = 1.0f,
    };

    Player player = {
        .screen = (Vector2) {window_size.x/2, window_size.y/2},
        .radius = 25.0f,
    };

    InitWindow(window_size.x, window_size.y, "voleur");
    SetWindowState(FLAG_WINDOW_RESIZABLE);
    SetTargetFPS(60);

    Map map = {
        .texture = LoadTexture("data/map2.png"),
        .screen = {-map.texture.width/2, -map.texture.height/2},
        .box = (Rectangle) {
            .x = map.screen.x, .y = map.screen.y,
            .width = map.texture.width, .height = map.texture.height,
        },
    };

    while (!WindowShouldClose()) {
        if (IsWindowResized()) {
            window_size = (Vector2) {GetScreenWidth(), GetScreenHeight()};
            camera.offset = (Vector2) {window_size.x/2, window_size.y/2};
            player.screen = (Vector2) {window_size.x/2, window_size.y/2};
        }

        float movement_speed = 10.0f;
        if ((IsKeyDown(KEY_LEFT) || IsKeyDown(KEY_RIGHT)) && (IsKeyDown(KEY_UP) || IsKeyDown(KEY_DOWN)))
            movement_speed = 6.324f;
        if (IsKeyDown(KEY_LEFT_SHIFT))
            movement_speed /= 2.0f;

        if (IsKeyDown(KEY_LEFT))
            camera.target.x -= movement_speed ;
        if (IsKeyDown(KEY_RIGHT))
            camera.target.x += movement_speed ;
        if (IsKeyDown(KEY_UP))
            camera.target.y -= movement_speed ;
        if (IsKeyDown(KEY_DOWN))
            camera.target.y += movement_speed ;

        if (IsKeyDown(KEY_A))
            camera.rotation++;
        if (IsKeyDown(KEY_D))
            camera.rotation--;
        if (IsKeyDown(KEY_W))
            player.radius++;
        if (IsKeyDown(KEY_S))
            player.radius--;

        player.coord = GetScreenToWorld2D(player.screen, camera);

        if (player.coord.x - player.radius < map.box.x)
            camera.target.x = map.box.x + player.radius;
        else if (player.coord.x + player.radius > map.box.x + map.box.width)
            camera.target.x = map.box.x + map.box.width - player.radius;

        if (player.coord.y - player.radius < map.box.y)
            camera.target.y = map.box.y + player.radius;
        else if (player.coord.y + player.radius > map.box.y + map.box.height)
            camera.target.y = map.box.y + map.box.height - player.radius;

        BeginDrawing();
        {
            ClearBackground(LIME);

            BeginMode2D(camera);
            {
                DrawTexture(map.texture, map.screen.x, map.screen.y, WHITE);
            }
            EndMode2D();

            DrawCircle(player.screen.x, player.screen.y, player.radius, RED);
        }
        EndDrawing();
    }

    CloseWindow();
    return 0;
}
